#! /usr/bin/perl
# 
#    Copyright 2006 Cybozu Labs, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

use strict;
use warnings;
use Digest::MD5 qw(md5_hex);


die "Usage: $0 <perl5webdb_dir> <port> <username> <password>\n" if @ARGV != 4;

sub generate_secret {
    my $fd;
    my $salt;
    open($fd, "<", "/dev/urandom") || die "cannot read from /dev/urandom";
    sysread($fd, $salt, 128) == 128 || die "failed to read from /dev/urandom";
    close($fd);
    return md5_hex(time . $$ . $salt);
}

my $webdb_dir = shift;
my $webdb_port = shift;
my $username = shift;
my $password = shift;

# generate secret
print STDERR "generating secret... ";
my $secret = generate_secret();
print STDERR "done\n";

# adjust .htpasswd file
print STDERR "creating/adjusting $webdb_dir/etc/passwd... ";
my $fd;
open($fd, ">>", "$webdb_dir/etc/passwd");
close($fd);
if (my $err = system("htpasswd -b '$webdb_dir/etc/passwd' '$username' '$password' > /dev/null 2>&1")) {
    die "failed to execute htpasswd: $!" if $err == -1;
    exit($err >> 8);
}
print STDERR "done\n";

# generate conf file
print <<"EOT";

SetEnv WEBDB_ROOT $webdb_dir
SetEnv WEBDB_PORT $webdb_port

SetEnvIf Cookie "webdb=$secret" PERL5OPT=-d PERL5DB=BEGIN{require'$webdb_dir/bin/perl5webdb.pl'}
SetEnvIf Request_URI "^/webdb/" !PERL5OPT

Alias /webdb/ "$webdb_dir/www/"
<Directory "$webdb_dir/www">
    AuthUserFile $webdb_dir/etc/passwd
    AuthName "WebDB Console"
    AuthType basic
    Require valid-user
    Options ExecCGI
    Order allow,deny
    Allow from all
    SetEnv WEBDB_SECRET=$secret
</Directory>

EOT

